#ifndef __BACKTEXTURES_FX_
#define __BACKTEXTURES_FX_
#include "ShaderHelper.fx"
#include "GlobalVariable.fx"
//--------------------------------------------------------------//
// BackDepth
//--------------------------------------------------------------//
struct VS_INPUT 
{
    float4 Position : POSITION0;
    float4 Normal: NORMAL0;

};

struct VS_OUTPUT 
{
    float4 Position : POSITION0;
    float3  Pos : TEXCOORD1;
};


VS_OUTPUT BackDepth_Vertex_Shader_vs_main( VS_INPUT Input )
{
    VS_OUTPUT Output;

    Output.Position = mul( Input.Position, matWorldViewProjection );
    Output.Pos = Input.Position;
    return( Output );

}


struct PS_OUTPUT
{
    float4 Position : COLOR0;
};

PS_OUTPUT BackDepth_Pixel_Shader_ps_main(VS_OUTPUT Input)
{
    PS_OUTPUT output;
    float4 pos = mul(float4(Input.Pos,1.0f), matWorldView);
    output.Position = (pos.z/depthscalar);
    return output;

}

//--------------------------------------------------------------//
// BackNormal
//--------------------------------------------------------------//


struct BackNormal_Vertex_Shader_VS_INPUT 
{
    float4 Position : POSITION0;
    float4 Normal: NORMAL0;

};

struct BackNormal_Vertex_Shader_VS_OUTPUT 
{
    float4 Position : POSITION0;
    float3 Normal: TEXCOORD0;
};


BackNormal_Vertex_Shader_VS_OUTPUT BackNormal_Vertex_Shader_vs_main( BackNormal_Vertex_Shader_VS_INPUT Input )
{
    BackNormal_Vertex_Shader_VS_OUTPUT Output;

    Output.Position = mul( Input.Position, matWorldViewProjection );
    Output.Normal = normalize(mul(float4(Input.Normal.xyz,0.0f), matWorldView )).xyz;
    return( Output );

}

struct BackNormal_Pixel_Shader_PS_OUTPUT
{
    float4 Normal : COLOR0;
};

BackNormal_Pixel_Shader_PS_OUTPUT BackNormal_Pixel_Shader_ps_main(BackNormal_Vertex_Shader_VS_OUTPUT Input)
{
    //Simply output normals
    BackNormal_Pixel_Shader_PS_OUTPUT output;
    output.Normal = float4 ( normalize(Input.Normal)*0.5f +  float3(0.5f,0.5f,0.5f), 1.0f);
    return output;

}

//--------------------------------------------------------------//
// BackRoughness
//--------------------------------------------------------------//
struct BackRoughness_Vertex_Shader_VS_INPUT 
{
    float4 Position : POSITION0;
    float4 Normal: NORMAL0;
    float4 TexCoord: TEXCOORD0;

};

struct BackRoughness_Vertex_Shader_VS_OUTPUT 
{
    float4 Position : POSITION0;
    float4 TexCoord: TEXCOORD0;
};


BackRoughness_Vertex_Shader_VS_OUTPUT BackRoughness_Vertex_Shader_vs_main( BackRoughness_Vertex_Shader_VS_INPUT Input )
{
    BackRoughness_Vertex_Shader_VS_OUTPUT Output;
    Output.Position = mul( Input.Position, matWorldViewProjection );
    Output.TexCoord = Input.TexCoord;
    return( Output );

}

struct BackRoughness_Pixel_Shader_PS_OUTPUT
{
    float4 Roughness : COLOR0;
};

BackRoughness_Pixel_Shader_PS_OUTPUT BackRoughness_Pixel_Shader_ps_main(BackRoughness_Vertex_Shader_VS_OUTPUT Input)
{
    //Simply output normals
    BackRoughness_Pixel_Shader_PS_OUTPUT output;
    output.Roughness = tex2D(texearth,Input.TexCoord)*roughnessscalar;
    return output;
}

#endif