#ifndef __OLIVEIRATEST_FX_
#define __OLIVEIRATEST_FX_

#include "ShaderHelper.fx"
#include "GlobalVariable.fx"
#include "Oliveiratestheader.fx"
//--------------------------------------------------------------//
// OliveiraTestPass
//--------------------------------------------------------------//

struct OliveiraTestPass_Vertex_Shader_VS_INPUT 
{
    float4 Position : POSITION0;
    float3 Normal : NORMAL0;
};

struct OliveiraTestPass_Vertex_Shader_VS_OUTPUT 
{
    float4 Position : POSITION0;
    float3 Normal : TEXCOORD0;
    float3 WorldPos : TEXCOORD1;
};


OliveiraTestPass_Vertex_Shader_VS_OUTPUT OliveiraTestPass_Vertex_Shader_vs_main( OliveiraTestPass_Vertex_Shader_VS_INPUT Input )
{
    OliveiraTestPass_Vertex_Shader_VS_OUTPUT Output;

    Output.Position = mul( Input.Position, matWorldViewProjection );
    Output.Normal = normalize(mul( float4(Input.Normal,0.0f), matWorldView )).xyz;
    Output.WorldPos = mul( Input.Position, matWorldView ).xyz;
    return( Output );

}





struct OliveiraTestPass_Pixel_Shader_PS_OUTPUT
{
    float4 color : COLOR0;
};


OliveiraTestPass_Pixel_Shader_PS_OUTPUT OliveiraTestPass_Pixel_Shader_ps_main(OliveiraTestPass_Vertex_Shader_VS_OUTPUT Input)
{   
    OliveiraTestPass_Pixel_Shader_PS_OUTPUT output;

    //normals
    float3 frontNormal = normalize(Input.Normal);

    //P1
    float3 P1 = Input.WorldPos;

    //refraction vector T1
    float3 view = normalize( - Input.WorldPos);
    float n1aboven2 = 1/N2;
    float w = n1aboven2 * max(0,dot(frontNormal, view));
    float k = sqrt(1 + w*w - n1aboven2*n1aboven2);
    float3 fraction = (w - k)*frontNormal - n1aboven2*view;
    float3 T1 = normalize(fraction);//T1 is right than never!

    //Get Oliveira's estimation
    float Zmax = (tex2D(texZmax,0.5f)).r*depthscalar;
    float Zmin = (tex2D(texZmin,0.5f)).r*depthscalar;

    float2 estimateCoord = Estimation_Olivera(P1, T1, Zmax, Zmin);
    //estimated normal, outgoin
    float3 backNormal = 
        -((tex2D(texBackNormal,estimateCoord).xyz - float3(0.5f,0.5f,0.5f))*2.0f);
    if (length(backNormal) < 0.00001f)
    {
        backNormal = -normalize(float3(frontNormal.xy,0));
    }
    else
    {
        backNormal = normalize(backNormal);
    }

    //Compute outgoing ray or second refraction
    //float maxCos = sqrt(1 - n1aboven2*n1aboven2);
    //w = 1/n1aboven2 * min(max(0,dot(backNormal, T1)),maxCos);
    //k = sqrt(max(0,1 + w*w - 1/n1aboven2/n1aboven2));
    //float3 outgoing = (w - k)*backNormal - 1/n1aboven2*T1;
    n1aboven2 = N2;
    w = n1aboven2 * dot(backNormal, -T1);
    k = sqrt(1 + w*w - n1aboven2*n1aboven2);
    fraction = (w - k)*backNormal + n1aboven2*T1;
    float3 outgoing = normalize(fraction);//T1 is right than never!

    //TODO:
    //outgoing = normalize(mul(float4(outgoing,0.0f), matViewInverse ));
    outgoing = normalize(mul(float4(outgoing,0.0f), matViewInverse ));


    output.color = EnvironmentMap(outgoing);
    //output.color = lerp(float4(backNormal,1),output.color, OliveiraTestPass_Pixel_Shader_depthscalar);

    //fresnel
    //refrection
    //float3 refrection = normalize(2*max(0,dot(view,frontNormal))*frontNormal - view);

    //output.color = lerp(output.color, texCUBE(Emap, refrection),tex2D(texFres, float2(max(0,dot(view,frontNormal)), max(0,dot(view,frontNormal)))).x);
    //output.color = tex2Dlod(texBackNormal,float4(estimateCoord,0,1));
    //output.color = abs(tex2Dlod(texBackNormal,float4(estimateCoord,0,8)) - tex2Dlod(texBackNormal,float4(estimateCoord,0,0)));

    //T1 = (T1 + 1)/2.0f;
    //output.color = float4(T1.x,T1.y, T1.z,1.0f);
    //output.color = Zmin/1500.0f;
    return output;

}

#endif