#ifndef __OLIVEIRATESTHEADER_FX_
#define __OLIVEIRATESTHEADER_FX_

#include "ShaderHelper.fx"
#include "GlobalVariable.fx"
//RefractedPoint is in *viewspace*, refractedRay is T1 also in view space
//   Notice DX's coordinate system is lefthanded, so the all depth should
//   >= 0
//   Return: the point that interseted in *viewspace*
inline float2 Estimation_Olivera(float3 P1, float3 T1, float Zmax, float Zmin)
{
    //Compute S and E
    T1 = normalize(T1);
    float3 S = P1;// + (Zmin-P1.z)*T1; //near
    float3 E = P1 + (Zmax-P1.z)/T1.z*T1; //far

    //Compute k1 and k2
    float k1 = 1/S.z;
    float k2 = (S.z*E.z)/(S.z - E.z);

    //Project S and E to texture coordinates, here I use the global 
    //    viewmatrix rudely, which should be passed in as parameter 
    //    of this function, notice the DX has lefthanded base rather
    //    than OpenGL as in Oliveira's paper
    float4 S_tilt = mul(float4(S,1.0f), matProjection);
    float2 t_s_astr = (float2(S_tilt.x,-S_tilt.y)/S_tilt.w)*0.5f + 0.5f 
        + float2(fInverseViewportWidth/2, fInverseViewportHeight/2); //ignoring the 1/2 pixel length
    float4 E_tilt = mul(float4(E,1.0f), matProjection);
    float2 t_e_astr = (float2(E_tilt.x,-E_tilt.y)/E_tilt.w)*0.5f + 0.5f
        + float2(fInverseViewportWidth/2, fInverseViewportHeight/2); //ignoring the 1/2 pixel length

    float2 d_t = (t_e_astr - t_s_astr);
    //now k1,k2,t_s_astr dt is ready for use!

    //initialize P2
    float3 P2 = S;
    float3 d_v = E-S;


    //Start the conservative binary search loop
    float alpha = 0.0f;
    float2 texCoord;
    float storedDepth = 0.0f;
    float PdotZ = 0.0f;
    for(int i = 0; i < CONSERVATIVE_STEP_NUMBER; ++i)
    {
        d_v = d_v*0.5f;
        PdotZ = P2.z+d_v.z;
        alpha = (1/(PdotZ) - k1)*k2;
        texCoord = t_s_astr + alpha*d_t;
        //sample the depth in *viewspace*
        storedDepth = (tex2D(texBackDepth, texCoord)).r*depthscalar;

        //Check if it is ok to advance, notice the case when
        //    the depthPixel is invalid, that is, zero.
        if( (PdotZ < storedDepth)&& storedDepth > 0.0001f )// && storedDepth > OliveiraTestPass_Pixel_Shader_DecodeFloatRGBA(float4(0,0,0,0)))
        {
            P2 = P2 + d_v;
        }

    }

    float4 estimatePos = mul( float4(P2,1), matProjection );
    float2 estimateCoord = float2((estimatePos.x/estimatePos.w + 1)/2 + fInverseViewportWidth/2
        ,(-estimatePos.y/estimatePos.w + 1)/2 + fInverseViewportHeight/2);

    return estimateCoord;

}
#endif