//**************************************************************//
//  Effect File exported by RenderMonkey 1.6
//
//  - Although many improvements were made to RenderMonkey FX  
//    file export, there are still situations that may cause   
//    compilation problems once the file is exported, such as  
//    occasional naming conflicts for methods, since FX format 
//    does not support any notions of name spaces. You need to 
//    try to create workspaces in such a way as to minimize    
//    potential naming conflicts on export.                    
//    
//  - Note that to minimize resulting name collisions in the FX 
//    file, RenderMonkey will mangle names for passes, shaders  
//    and function names as necessary to reduce name conflicts. 
//**************************************************************//
#include "GlobalVariable.fx"
#include "ShaderHelper.fx"
#include "BackTextures.fx"
#include "MinMaxDepth.fx"
#include "Background.fx"
#include "OliveiraTest.fx"
#include "SplitMerge.fx"
//--------------------------------------------------------------//
// Rough Refraction
//--------------------------------------------------------------//
//TODO: reorganize this including files(yet I dunno
//  if effect compiler would support...
//--------------------------------------------------------------//
// FinalPass
//--------------------------------------------------------------//
struct FinalPass_Vertex_Shader_VS_INPUT 
{
    float4 Position : POSITION0;
    float3 Normal : NORMAL0;
    float4 TexCoord : TEXCOORD0;
};

struct FinalPass_Vertex_Shader_VS_OUTPUT 
{
    float4 Position : POSITION0;
    float3 Normal : TEXCOORD0;
    float3 WorldPos : TEXCOORD1;
    float4 TexCoord : TEXCOORD2;
    float2 RenderTarget: TEXCOORD3;
};


FinalPass_Vertex_Shader_VS_OUTPUT FinalPass_Vertex_Shader_vs_main( FinalPass_Vertex_Shader_VS_INPUT Input )
{
    FinalPass_Vertex_Shader_VS_OUTPUT Output;

    Output.Position = mul( Input.Position, matWorldViewProjection );
    Output.RenderTarget = float2((Output.Position.x/Output.Position.w + 1)/2 + fInverseViewportWidth/2
        ,(-Output.Position.y/Output.Position.w + 1)/2 + fInverseViewportHeight/2);
    Output.Normal = normalize(mul( float4(Input.Normal,0.0f), matWorldView )).xyz;
    Output.WorldPos = mul( Input.Position, matWorldView ).xyz;
    Output.TexCoord = Input.TexCoord;
    return( Output );

}

struct FinalPass_Pixel_Shader_PS_OUTPUT
{
    float4 color : COLOR0;
};

sampler2D tex1stmerge = sampler_state
{   
    Texture = (back_1stMergeTex);
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
    ADDRESSU = CLAMP;
    ADDRESSV = CLAMP;
};
sampler2D tex2ndmerge = sampler_state
{   
    Texture = (back_2ndMergeTex);
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
    ADDRESSU = CLAMP;
    ADDRESSV = CLAMP;
};
sampler2D tex3rdmerge = sampler_state
{   
    Texture = (back_3rdMergeTex);
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
    ADDRESSU = CLAMP;
    ADDRESSV = CLAMP;
};
sampler2D tex4thmerge = sampler_state
{   
    Texture = (back_4thMergeTex);
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
    ADDRESSU = CLAMP;
    ADDRESSV = CLAMP;
};

//In fact pt is the -dot( normal.hatpt), notice every input vector should be
//  normalized before you call this function!!! and dot(normal.view)>0 AND 
//  MAKE SURE *pt > 0* !!!!!
inline float3 ComputeHatPt(float pt, float3 normal, float3 view)
{
    //This formula -v+s*n can't do well because of float precision
    //PROBLEM: this problem is due to float computation unstability
    //SOLUTION: redesign a formula like s*v + (1-s)*n, leaving this float
    //  computation to matlab's precomputation, but we should convert it to 
    //  [0 1], magic number is used to convert [0, 1] to its orignal range
    pt = (pt*10.0f - 0.5f);
    return normalize(-(pt*view + (1-pt)*normal));
}
//The same as Estimation_Olivera, this function require all the vector
//  be normalized and in *viewspace*!
inline float4 ComputeDxDy(float3 P1, float3 hatPt, float cosThetaB, float Zmax, float Zmin)
{
    const float3 xaxis = float3(1,0,0);
    const float3 yaxis = float3(0,1,0);
    hatPt = normalize(hatPt);
    //Compute t in my formula
    float cossq = cosThetaB*cosThetaB;
    float2 xyt = hatPt.xy;
    float2 a = cossq - xyt*xyt;
    //if(a.x < 0.00001f || a.y < 0.00001f)
    //{
    //    return float4(fInverseViewportWidth,fInverseViewportHeight,
    //        fInverseViewportWidth,fInverseViewportHeight);
    //}
    //TODO:paralell?
    float2 b = 2*(cossq-1)*xyt;
    float c = cossq-1;
    float2 maxt = (-b +sqrt(max(0,b*b-4*a*c)))/a/2;
    float2 mint = (-b -sqrt(max(0,b*b-4*a*c)))/a/2;
    //find the intersection
    float2 maxx = Estimation_Olivera(P1, normalize(hatPt + xaxis*maxt.x), Zmax, Zmin );
    float2 maxy = Estimation_Olivera(P1, normalize(hatPt + yaxis*maxt.y), Zmax, Zmin );
    float2 minx = Estimation_Olivera(P1, normalize(hatPt + xaxis*mint.x), Zmax, Zmin );
    float2 miny = Estimation_Olivera(P1, normalize(hatPt + yaxis*mint.y), Zmax, Zmin );
    float2 ddx = (maxx-minx);//roughnessscalar;//(maxx-minx);
    float2 ddy = (miny-maxy);//roughnessscalar;//(miny-maxy);
    //bounding dx dy
    return float4(ddx.x,ddx.y,ddy.x,ddy.y);
}
//Compute the color of one merged lobe, backGaussian is the average gaussian represetnative
//   ,it has mu, kappa and alpha which are all in [0 1]
inline float4 ComputeGausianLobeColor(float4 backGaussian, float4 frontGaussian, float3 view)
{
    //convoluted alpha & kappa(roughness) merged.xy = mu merged.z= alpha
    // To make it beautiful we artificially disable the frontalpha = 1;
    float alpha = ( backGaussian.z * backMaxAlphaKappa.x);
    backGaussian.w = backGaussian.w*backMaxAlphaKappa.y;
    frontGaussian.y = frontGaussian.y*frontMaxAlphaKappa.y;
    float roughness = backGaussian.w*frontGaussian.y/(backGaussian.w+frontGaussian.y);
    //convert roughness to [0, 1]
    roughness = min(1, roughness/roughnessMax);
    backGaussian.xy = backGaussian.xy*2.0f - 1.0f;
    float3 proxyNormal = normalize( float3(backGaussian.xy
        ,sqrt(max(0.00001f,1-dot(backGaussian.xy,backGaussian.xy)) )));

    //use proxy Normal to index the back 
    float cosFrontNV = dot(proxyNormal, view);
    //TODO: Change to texbackBTDF and 0.5
    float2 alphaKappaPtTexCoord = float2(roughness,max(0.0f,cosFrontNV));
    float4 back = tex2Dlod(texbackBTDF,float4(alphaKappaPtTexCoord,0,2));
    float3 hatpt = ComputeHatPt(back.z, proxyNormal, view);

    //return color        
    hatpt = -normalize(mul(float4(hatpt,0.0f), matViewInverse ));
    //TODO: what about back.z
    //TODO: magic no
    //return  alpha*texCUBElod(Emap,float4(hatpt, max(0,1/back.y/3.5f)));
    return  alpha*EnvironmentMap(hatpt, back.y);

}
const float DECODED_TEXTURE_CONSTANT = 10.0f;
FinalPass_Pixel_Shader_PS_OUTPUT FinalPass_Pixel_Shader_ps_main(FinalPass_Vertex_Shader_VS_OUTPUT Input)
{   
    FinalPass_Pixel_Shader_PS_OUTPUT output;
    //roughness, here simply use red channel
    //Get Oliveira's Min Max estimation
    float Zmax = (tex2D(texZmax,0.5f)).r*depthscalar;
    float Zmin = (tex2D(texZmin,0.5f)).r*depthscalar;
    //Front Roughness
    float roughness = max(0.01f,tex2D(texearth,Input.TexCoord).r*roughnessscalar + roughnessaddition);
    //normal and view
    float3 frontNormal = normalize(Input.Normal);
    float3 view = normalize( - Input.WorldPos);
    float cosFrontNV = dot(frontNormal, view);

    //get alpha and kappa and pt
    float2 alphaKappaPtTexCoord = float2(roughness,max(0.0f,cosFrontNV)); 
    //Lod to smooth the precomputation from matlab, mipmap box filter is good enough
    //Comment this if we have a better precomputed texture
    float4 frontBTDF = tex2Dlod(texfrontBTDF,float4(alphaKappaPtTexCoord,0,2));
    float3 hatpt = ComputeHatPt(frontBTDF.z, frontNormal, view);
    //DxDy bounding
    float3 P1 = Input.WorldPos;
    float4 dd = ComputeDxDy(P1, hatpt,frontBTDF.w,Zmax,Zmin);
    //Load texture with this dd
    //Load backnormal NOTICE THE MINUS, normal in texture is z positive in DX
    float2 texCoord = Estimation_Olivera(P1, hatpt, Zmax, Zmin );
    //Load the quadrant
    float4 color1stMerge = tex2Dgrad(tex1stmerge,texCoord, dd.xy,dd.zw);
    float4 color2ndMerge = tex2Dgrad(tex2ndmerge,texCoord, dd.xy,dd.zw);
    float4 color3rdMerge = tex2Dgrad(tex3rdmerge,texCoord, dd.xy,dd.zw);
    float4 color4thMerge = tex2Dgrad(tex4thmerge,texCoord, dd.xy,dd.zw);

    output.color = 0;
    output.color += ComputeGausianLobeColor(color1stMerge,float4(frontBTDF.xy,0,0),hatpt);
    output.color += ComputeGausianLobeColor(color2ndMerge,float4(frontBTDF.xy,0,0),hatpt);
    output.color += ComputeGausianLobeColor(color3rdMerge,float4(frontBTDF.xy,0,0),hatpt);
    output.color += ComputeGausianLobeColor(color4thMerge,float4(frontBTDF.xy,0,0),hatpt);

    return output;

}
inline float3 ConvoluteGaussian(float2 mean, float3 moment, float s)
{
    float3 devMatrix = 0;
    devMatrix.x = moment.x-mean.x*mean.x + 1.0f/s;
    devMatrix.y = moment.y-mean.x*mean.y;
    devMatrix.z = moment.z-mean.y*mean.y + 1.0f/s;
    //Compute Inverse
    devMatrix = ComputeSymmetricMatrixInverse(devMatrix);
    return devMatrix;
}
inline float4 BackGaussianCubeMap(float3 proxyNormal, float3 view, float roughness)
{
    //use proxy Normal to index the back 
    float cosFrontNV = dot(proxyNormal, view);
    //TODO: Change to texbackBTDF and 0.5
    float2 alphaKappaPtTexCoord = float2(roughness,max(0.0f,cosFrontNV));
    float4 back = tex2Dlod(texbackBTDF,float4(alphaKappaPtTexCoord,0,2));
    float3 hatpt = ComputeHatPt(back.z, proxyNormal, view);

    //return color        
    hatpt = -normalize(mul(float4(hatpt,0.0f), matViewInverse ));
    //TODO: what about back.z
    //TODO: magic no
    //return  alpha*texCUBElod(Emap,float4(hatpt, max(0,1/back.y/3.5f)));
    return  EnvironmentMap(hatpt,back.y);
}
inline float4 ComputeEWAColor(float3 gaussianMatrix, float2 mean, float3 hatpt)
{
    const float alpha = 1.0f;
    //Compute r Rmajor Rminor
    gaussianMatrix.y = 2*gaussianMatrix.y;
    gaussianMatrix = max(gaussianMatrix, 0.001f);
    float r = sqrt((gaussianMatrix.x - gaussianMatrix.z)*(gaussianMatrix.x - gaussianMatrix.z)
        + gaussianMatrix.y*gaussianMatrix.y);
    float Rmajor = sqrt(2.0f/(gaussianMatrix.x + gaussianMatrix.z - r));
    float Rminor = sqrt(2.0f/(gaussianMatrix.x + gaussianMatrix.z + r));

    //Compute Nprobe and L
    const int loop = 3;//max(1,floor(Rmajor/Rminor/alpha));
    int Nprobe = 2*loop - 1;
    float L = (Rmajor- Rminor);//2.0f*(Rmajor - Rminor*alpha);

    //theta
    float theta = 0;
    if(abs(gaussianMatrix.x - gaussianMatrix.z) < 0.0001f)
    {
        L = 0.0f;
    }
    else
    {
        theta = atan(gaussianMatrix.y/(gaussianMatrix.x - gaussianMatrix.z))/2.0f;
    }

    L = L/(loop - 1);
    float2 duv = float2(cos(theta)*L,sin(theta)*L);
    float2 projNormal = mean;
    float roughness = max((gaussianMatrix.x),(gaussianMatrix.z));
    roughness = max(0.001f, roughness/50.0f);
    float4 color = BackGaussianCubeMap(RestoreProjectedNormal(mean).xyz, hatpt, roughness);

    for( int i = 1; i < loop; ++i)
    {
        color += BackGaussianCubeMap(RestoreProjectedNormal(max(-0.99f,min(0.99f,mean + i*duv))).xyz, hatpt, roughness);
        color += BackGaussianCubeMap(RestoreProjectedNormal(max(-0.99f,min(0.99f,mean - i*duv))).xyz, hatpt, roughness);
    }

    return color/5.0f;//BackGaussianCubeMap(RestoreProjectedNormal(mean).xyz, hatpt, roughness);

}
FinalPass_Pixel_Shader_PS_OUTPUT LeanFinalPass_Pixel_Shader_ps_main(FinalPass_Vertex_Shader_VS_OUTPUT Input)
{   
    FinalPass_Pixel_Shader_PS_OUTPUT output;
    //roughness, here simply use red channel
    //Get Oliveira's Min Max estimation
    float Zmax = (tex2D(texZmax,0.5f)).r*depthscalar;
    float Zmin = (tex2D(texZmin,0.5f)).r*depthscalar;
    //Front Roughness
    float roughness = tex2D(texearth,Input.TexCoord).r*roughnessscalar + roughnessaddition;
    //normal and view
    float3 frontNormal = normalize(Input.Normal);
    float3 view = normalize( - Input.WorldPos);
    float cosFrontNV = dot(frontNormal, view);

    //get alpha and kappa and pt
    float2 alphaKappaPtTexCoord = float2(roughness,max(0.0f,cosFrontNV)); 
    //Lod to smooth the precomputation from matlab, mipmap box filter is good enough
    //Comment this if we have a better precomputed texture
    float4 frontBTDF = tex2Dlod(texfrontBTDF,float4(alphaKappaPtTexCoord,0,2));
    float3 hatpt = ComputeHatPt(frontBTDF.z, frontNormal, view);
    //DxDy bounding
    float3 P1 = Input.WorldPos;
    float4 dd = ComputeDxDy(P1, hatpt,frontBTDF.w,Zmax,Zmin);
    //Load texture with this dd
    //Load backnormal NOTICE THE MINUS, normal in texture is z positive in DX
    float2 texCoord = Estimation_Olivera(P1, hatpt, Zmax, Zmin );
    //Load the quadrant
    float2 Lean1 = DecodeNormal2d(tex2Dgrad(tex1stmerge,texCoord, dd.xy,dd.zw).xy);
    float3 Lean2 = DecodeNormal3d(tex2Dgrad(tex2ndmerge,texCoord, dd.xy,dd.zw).xyz)*normalMax;
   
    float3 moment = float3(Lean2.x,Lean2.y,Lean2.z);

    //Convolute Gaussian
    float3 convolutedGaussian = ConvoluteGaussian(Lean1,moment,max(0.0001f,frontMaxAlphaKappa.y*frontBTDF.y));

    ////Compute EWA color
    output.color = 0;
    output.color = ComputeEWAColor(convolutedGaussian,Lean1,hatpt);

    //output.color /= 4.0f;
    //TODO:
    //output.color = frontBTDF.z *60;
    //hatpt = normalize(mul(float4(hatpt,0.0f), matViewInverse )).xyz;
    //output.color = EnvironmentMap(hatpt, 1);
    //output.color = texCUBEgrad(Emap,hatpt,dd.xyy,dd.zww);
    //output.color = float4((hatpt+1)/2.0f,1);
    //output.color = float4(meanBackNormal,1);
    //output.color = tex2Dgrad(texBackNormal,texCoord, dd.xy,dd.zw);
    //output.color = float4(tex2Dlod(tex1stmerge,float4(texCoord,0,roughnessscalar*8)).xy,0,0);
    //output.color = 0;
    //float3 temp = Lean2  - float3(Lean1.x*Lean1.x,Lean1.x*Lean1.y,Lean1.y*Lean1.y);
    ////temp += float3(1/frontMaxAlphaKappa.y*frontBTDF.y,0,1/frontMaxAlphaKappa.y*frontBTDF.y);
    ////output.color += float4(Lean2-float3(Lean1.x*Lean1.x,Lean1.x*Lean1.y,Lean1.y*Lean1.y),1);
    //float x = (temp.x*temp.z - temp.y*temp.y);
    ////output.color += (temp.x*temp.z - temp.y*temp.y)*2.0f;
    ////if(x > 0.0001f)
    ////{
    //temp = temp/max(0,abs(temp.x*temp.z - temp.y*temp.y));
    //output.color += max(temp.x,temp.z)/50.0f;//+float4(x,1)/1.0f;//
    //}
    //else
    //{
    //    temp += float3(inversekappa,0,inversekappa);
    //temp = temp/max(0.0001f,abs(temp.x*temp.z - temp.y*temp.y))*sign(temp.x*temp.z - temp.y*temp.y);
    //    float y = min((0.0001f-x)/0.00005f,1);

    //    output.color = lerp(1/inversekappa/500.0f,(temp.x+temp.z)/2.0f/500.0f,1-y);
    //}
    //if(temp.x > 0.1f || temp.z > 0.1f)
    //    output.color = float4(0,0,1,1);
    //else
    //    ;
    //output.color = float4(temp*1000,0);
    //output.color += tex2Dgrad(tex2ndmerge,texCoord, dd.xy,dd.zw).z;
    //output.color += tex2Dgrad(tex3rdmerge,texCoord, dd.xy,dd.zw).z;
    //output.color += tex2Dgrad(tex4thmerge,texCoord, dd.xy,dd.zw).z;
    //output.color /= 2.0f;
    //output.color = tex2D(tex1stmerge,texCoord);
    //output.color = tex2Dgrad(texBackNormal,Estimation_Olivera(P1, hatpt, Zmax, Zmin ), dd.xy,dd.zw);
    //output.color = tex2D(texpt,alphaKappaPtTexCoord).a;

    //cosFrontNV = dot(meanBackNormal, -hatpt); //n > 0  
    //alphaKappaPtTexCoord = float2(1,cosFrontNV);
    //frontBTDF = tex2Dlod(texbackBTDF,float4(alphaKappaPtTexCoord,0,2));
    //hatpt = ComputeHatPt(frontBTDF.z, meanBackNormal, -hatpt);
    //hatpt = normalize(mul(float4(hatpt,0.0f), matViewInverse ));
    //output.color = texCUBElod(Emap,float4(hatpt, 0));
    ////output.color = dot(meanBackNormal, -hatpt)*0.8f;
    return output;

}
//--------------------------------------------------------------//
// Technique Section for Effect Workspace.Wyman.Olivera
//--------------------------------------------------------------//
technique Olivera
{
    pass BackDepth
        //<
        //string Script = "RenderColorTarget0 = back_Tex;"
        //"ClearColor = (0, 0, 0, 255);"
        //"ClearDepth = 1.000000;";
        //>
    {
        ZFUNC = GREATER;
        CULLMODE = CW;
        ALPHABLENDENABLE = TRUE;

        VertexShader = compile vs_2_0 BackDepth_Vertex_Shader_vs_main();
        PixelShader = compile ps_2_0 BackDepth_Pixel_Shader_ps_main();
    }

    pass BackNormal
        //<
        //string Script = "RenderColorTarget0 = backNormal_Tex;"
        //"ClearColor = (127, 127, 127, 255);"
        //"ClearDepth = 1.000000;";
        //>
    {
        ZFUNC = GREATER;
        CULLMODE = CW;
        ALPHABLENDENABLE = TRUE;
        VertexShader = compile vs_2_0 BackNormal_Vertex_Shader_vs_main();
        PixelShader = compile ps_2_0 BackNormal_Pixel_Shader_ps_main();
    }

    pass BackRoughness
        //<
        //string Script = "RenderColorTarget0 = backNormal_Tex;"
        //"ClearColor = (127, 127, 127, 255);"
        //"ClearDepth = 1.000000;";
        //>
    {
        ZFUNC = GREATER;
        CULLMODE = CW;
        ALPHABLENDENABLE = TRUE;
        VertexShader = compile vs_2_0 BackRoughness_Vertex_Shader_vs_main();
        PixelShader = compile ps_2_0 BackRoughness_Pixel_Shader_ps_main();
    }

    pass MinMax
    {
        ZFUNC = LESSEQUAL;
        ALPHABLENDENABLE = TRUE;
        VertexShader = compile vs_3_0 MinMax_Vertex_Shader_vs_main();
        PixelShader = compile ps_3_0 MinMax_Pixel_Shader_ps_main();
    }

    pass Background
    {
        CULLMODE = NONE;
        ZFUNC = LESSEQUAL;
        ALPHABLENDENABLE = FALSE;

        VertexShader = compile vs_3_0 Background_Vertex_Shader_vs_main();
        PixelShader = compile ps_3_0 Background_Pixel_Shader_ps_main();
    }

    pass OliveiraTestPass
    {
        ZFUNC = LESSEQUAL;
        CULLMODE = CCW;
        ALPHABLENDENABLE = TRUE;

        VertexShader = compile vs_3_0 OliveiraTestPass_Vertex_Shader_vs_main();
        PixelShader = compile ps_3_0 OliveiraTestPass_Pixel_Shader_ps_main();
    }

    pass Split
    {
        ZFUNC = LESSEQUAL;
        ALPHABLENDENABLE = TRUE;
        VertexShader = compile vs_3_0 Split_Vertex_Shader_vs_main();
        PixelShader = compile ps_3_0 Split_Pixel_Shader_ps_main();
    }
    pass Lean
    {
        ZFUNC = LESSEQUAL;
        ALPHABLENDENABLE = TRUE;
        VertexShader = compile vs_3_0 Lean_Vertex_Shader_vs_main();
        PixelShader = compile ps_3_0 Lean_Pixel_Shader_ps_main();
    }

    pass Merge
    {
        ZFUNC = LESSEQUAL;
        ALPHABLENDENABLE = TRUE;
        VertexShader = compile vs_3_0 Merge_Vertex_Shader_vs_main();
        PixelShader = compile ps_3_0 Merge_Pixel_Shader_ps_main();
    }
    pass LeanDownsample
    {
        ZFUNC = LESSEQUAL;
        ALPHABLENDENABLE = TRUE;
        VertexShader = compile vs_3_0 LeanDownsample_Vertex_Shader_vs_main();
        PixelShader = compile ps_3_0 LeanDownsample_Pixel_Shader_ps_main();
    }

    pass FinalPass
    {
        ZFUNC = LESSEQUAL;
        CULLMODE = CCW;
        ALPHABLENDENABLE = TRUE;

        VertexShader = compile vs_3_0 FinalPass_Vertex_Shader_vs_main();
        PixelShader = compile ps_3_0 FinalPass_Pixel_Shader_ps_main();
    }
    pass LeanFinalPass
    {
        ZFUNC = LESSEQUAL;
        CULLMODE = CCW;
        ALPHABLENDENABLE = TRUE;

        VertexShader = compile vs_3_0 FinalPass_Vertex_Shader_vs_main();
        PixelShader = compile ps_3_0 LeanFinalPass_Pixel_Shader_ps_main();
    }
}

