#ifndef __SHADERHELPER_FX_
#define __SHADERHELPER_FX_
//NOTICE PACKING IS USED FOR FLOAT IN [0 1]!!!!!
//PROBLEM: the packing need computation, more texture addressing
//  and lack precision and most importantly bring me too much trouble 
//  in bug shooting. use this shader only for old video card.
//SOLUTION: use float32 texture instead.
inline float4 EncodeFloatRGBA( float v ) {
    float4 enc = float4(1.0, 255.0, 65025.0, 160581375.0) * v;
    enc = frac(enc);
    enc -= enc.yzww * float4(1.0/255.0,1.0/255.0,1.0/255.0,0.0);
    return enc;
}
inline float DecodeFloatRGBA( float4 rgba ) {
    return dot( rgba, float4(1.0, 1/255.0, 1/65025.0, 1/160581375.0) );
}

//This function returns the Parameter of a Gaussian Density funciton approximate NDF.
//  NOTICE: roughness is in [0 1] and also return
inline float2 ParameterOfBackGaussian(float roughness)
{
    roughness = roughness*roughnessMax;
    return float2(1.0/backMaxAlphaKappa.x, roughness/backMaxAlphaKappa.y);//
}
inline float4 RestoreProjectedNormal(float2 normal2d)
{
    //If the normal is invalid
    float sqrlen = dot(normal2d,normal2d);
    if ( sqrlen> 1)
    {
        return 0;//float4(normalize(float3(normal2d,0.0f)),0.0f);
    }
    else
    {
        return float4(normal2d.xy,sqrt(1-sqrlen),1);
    }
}
inline float2 ProjectNormal(float3 normal3d)
{
    //If the normal is invalid
    if(dot(normal3d,normal3d)<0.0001f)
    {
        return INVALID_PROJECTED_NORMAL;
    }
    else
    {
        normal3d = normalize(normal3d);
        return normal3d.xy;
    }
}
////TODO: 2 project normal function
//inline float2 ProjectNormalLean(float3 normal3d)
//{
//    //If the normal is invalid
//    if(abs(normal3d.z) <0.001f)
//    {
//        if(normal3d.z >= 0)
//        {
//            return normal3d.xy/0.001f; //0
//        }
//        else
//        {
//            return -normal3d.xy/0.001f;
//        }
//    }
//    else
//    {
//        return normal3d.xy/normal3d.z; 
//    }
//}
//inline float3 RestoreProjectNormalLean(float2 normal2d)
//{
//    if(normal2d.x > normalMax || normal2d.y > normalMax)
//    {
//        return normalize(float3(normal2d.x/normalMax,normal2d.y/normalMax,0));
//    }
//    else
//    {
//        return normalize(float3(normal2d,1));
//    }
//}
inline float3 ComputeSymmetricMatrixInverse(float3 symMatrix)
{
    const float epsilon = 0.0001f;
    //A=	symMatrix.x	symMatrix.y
    //      symMatrix.y	symMatrix.z
    //TODO: det might be zer
    float det = symMatrix.x*symMatrix.z - symMatrix.y*symMatrix.y;
    float3 inverse = float3(symMatrix.z, -symMatrix.y, symMatrix.x)/max(0.0001f,abs(det))*sign(det);
    
    return inverse;
}
//For normal in [-1 1]x[-1 1]x[-1 1]
//TODO: macro or template?
inline float3 EncodeNormal3d(float3 normal)
{
    return (normal+1)/2.0f;
}
inline float3 DecodeNormal3d(float3 normal)
{
    return (normal*2.0f -1);
}

inline float2 EncodeNormal2d(float2 normal)
{
    return (normal+1)/2.0f;
}
inline float2 DecodeNormal2d(float2 normal)
{
    return (normal*2.0f -1);
}
//Return the scalar of a standard Gaussian (normalized)
//inline float ScalarOfGaussian(float alpha, float kappa)
//{
//    return alpha*doublePi/kappa;
//}
//inversedeviation is in [0 1] 
inline float2 CylinderMap(float3 vec)
{
    vec = normalize(vec);
    float t = 1-(vec.y + 1)/2.0f;
    float s = 0;
    float x = vec.x*vec.x + vec.z*vec.z;
    if (x > 0.0001f)
    {
        x = vec.x/(sqrt(x));
        if ( vec.z >= 0)
        {
            s = acos(x)/doublePi;
            //return float4(1,0,0,1);
        }
        else
        {
            s = 1 - acos(x)/doublePi;
    
            //return float4(0,1,0,1);
        }
    }
    else
    {
        s = 0;
        //return float4(1,0,0,1);
    }
    return float2(s,t);
}
//inline float4 EnvironmentMap(float3 vec, float inversedeviation)
//{
//    float2 uv = CylinderMap(vec);
//    return tex3D(texvol,float3(uv,inversedeviation));
//}

//     ^
//     |
// 6 | 7 | 8 
//-----------
// 5 | 9 | 1 --->
//-----------
// 4 | 3 | 2 
//
const float2 gaussianOffset[9] = {
    float2(1, 0),
    float2(0.7071067f, -0.7071067f),
    float2(0, -1),
    float2(-0.7071067f,-0.7071067f),

    float2(-1,0),
    float2(-0.7071067f,0.7071067f),
    float2(0, 1),
    float2(0.7071067f,0.7071067f),

    float2(0,0)
};
const float gaussianWeight[9] = {
    0.0838,
    0.0113,
    0.0838,
    0.0113,

    0.0838,
    0.0113,
    0.0838,
    0.0113,

    0.6193
};
const float k_delta = 2.0f/3.0f;
inline float4 EnvironmentMap(float3 vec, float inversedeviation = 1.0f)
{
    float2 uv = CylinderMap(vec);
    float4 color = 0;

    if(!preconvolutedEnvMap)
    {
        float m = max(0.0f,log(k_delta/inversedeviation)/log(2.0f));
        //int l = floor(m);
        for(int i = 0; i < 9; ++i)
        {
            color += tex2Dlod(texcylinder,float4(uv+gaussianOffset[i]*sign(m),0.0f,m))*gaussianWeight[i];
        }
    }
    else
    {
        color = tex3D(texvol,float3(uv,inversedeviation));
    }
    return color;
}
#endif
 