#ifndef __SPLITMERGE_FX_
#define __SPLITMERGE_FX_

#include "ShaderHelper.fx"
#include "GlobalVariable.fx"
//--------------------------------------------------------------//
// Split
//--------------------------------------------------------------//
struct Split_Vertex_Shader_VS_INPUT 
{
    float4 Position : POSITION0;
};

struct Split_Vertex_Shader_VS_OUTPUT 
{
    float4 Position : POSITION0;
    float2 texCoord : TEXCOORD0;
};

Split_Vertex_Shader_VS_OUTPUT Split_Vertex_Shader_vs_main( Split_Vertex_Shader_VS_INPUT Input )
{
    Split_Vertex_Shader_VS_OUTPUT Output;

    Output.Position = Input.Position;//mul( Input.Position, Split_Vertex_Shader_matViewProjection );
    Output.texCoord = float2((Output.Position.x/Output.Position.w + 1)/2 + fInverseViewportWidth/2
        ,(-Output.Position.y/Output.Position.w + 1)/2 + fInverseViewportHeight/2);

    return( Output );

}

struct Split_Pixel_Shader_PS_OUTPUT
{
    float4 FirstColor : COLOR0;
    float4 SecondColor : COLOR1;
    float4 ThirdColor : COLOR2;
    float4 FourthColor : COLOR3;
};

Split_Pixel_Shader_PS_OUTPUT Split_Pixel_Shader_ps_main(Split_Vertex_Shader_VS_OUTPUT input)
{ 
    Split_Pixel_Shader_PS_OUTPUT output;
    float roughness =max(0.01f,tex2D(texbackroughness,input.texCoord).r*roughnessscalar + roughnessaddition);
    //bakcNormal
    float3 color = tex2D(texBackNormal,input.texCoord).xyz;
    float3 backNormal = DecodeNormal3d(color); 
    
    float4 clearColor = 0;
    clearColor.xy = EncodeNormal2d(INVALID_PROJECTED_NORMAL);
    clearColor.z = 0.15f;
    output.FirstColor = clearColor;
    output.SecondColor = clearColor;
    output.ThirdColor = clearColor;
    output.FourthColor = clearColor;

    //TODO: magic number?
    //If normal is 0
    if(length(backNormal) < 0.01f )
    {
        return output;
    }
    else
    {
        backNormal = normalize(backNormal);

        //Get alpha and kappa based on roughness
        float2 backalphakappa = ParameterOfBackGaussian(roughness);
        float alpha = backalphakappa.x;
        float kappa = backalphakappa.y;
        //output.FirstColor.z = alpha;
        //output.FirstColor.w = kappa;

        const float cacheAreaDistance = 0.5f;
        float s = min(1,max(0,(backNormal.x/cacheAreaDistance + 1)/2.0f));
        float t = min(1,max(0,(backNormal.y/cacheAreaDistance + 1)/2.0f));
        float4 scalar = float4((1-s)*t, s*t, (1-t)*s, (1-s)*(1-t));
        //s, t should be in [0 1] 
        scalar = scalar/dot(scalar,1);
        output.FirstColor.xy = EncodeNormal2d(backNormal.xy); //n2*scalar.y
        output.SecondColor.xy =EncodeNormal2d(backNormal.xy); //n1*scalar.x
        output.ThirdColor.xy = EncodeNormal2d(backNormal.xy); //n4*scalar.z
        output.FourthColor.xy =EncodeNormal2d(backNormal.xy); //n3*scalar.w
        output.FirstColor.z = alpha*scalar.y;
        output.SecondColor.z = alpha*scalar.x;
        output.ThirdColor.z = alpha*scalar.z;
        output.FourthColor.z = alpha*scalar.w;
        output.FirstColor.w = kappa;
        output.SecondColor.w = kappa;
        output.ThirdColor.w = kappa;
        output.FourthColor.w = kappa;

        return output;
    }

}

//--------------------------------------------------------------//
// Lean
//--------------------------------------------------------------//
struct Lean_Vertex_Shader_VS_INPUT 
{
    float4 Position : POSITION0;
};

struct Lean_Vertex_Shader_VS_OUTPUT 
{
    float4 Position : POSITION0;
    float2 texCoord : TEXCOORD0;
};

Lean_Vertex_Shader_VS_OUTPUT Lean_Vertex_Shader_vs_main( Lean_Vertex_Shader_VS_INPUT Input )
{
    Lean_Vertex_Shader_VS_OUTPUT Output;

    Output.Position = Input.Position;//mul( Input.Position, Lean_Vertex_Shader_matViewProjection );
    Output.texCoord = float2((Output.Position.x/Output.Position.w + 1)/2 + fInverseViewportWidth/2
        ,(-Output.Position.y/Output.Position.w + 1)/2 + fInverseViewportHeight/2);

    return( Output );

}

struct Lean_Pixel_Shader_PS_OUTPUT
{
    float4 Lean1 : COLOR0;
    float4 Lean2 : COLOR1;
};

Lean_Pixel_Shader_PS_OUTPUT Lean_Pixel_Shader_ps_main(Lean_Vertex_Shader_VS_OUTPUT input)
{ 
    Lean_Pixel_Shader_PS_OUTPUT output;
    float roughness =tex2D(texbackroughness,input.texCoord).r*roughnessscalar + roughnessaddition;
    //roughness = min(1.0f,roughness);
    //bakcNormal
    float3 color = tex2D(texBackNormal,input.texCoord).xyz;
    float3 backNormal = DecodeNormal3d(color); 
    
    float4 clearColor = float4(1,0,1,1);
    //clearColor.xy = 0;
    //clearColor.z = 0.15f;
    //output.Lean1 = float4(EncodeNormal3d(clearColor.xyz),0);
    output.Lean1 =  float4(EncodeNormal2d(0),0,0);
    output.Lean2 = float4(EncodeNormal3d(clearColor.xyz),0);


    //TODO: magic number? precision?
    //If normal is 0
    if(length(backNormal) < 0.01f )
    {
        return output;
    }
    else
    {
        backNormal = normalize(backNormal);

        //Get alpha and kappa based on roughness
        float2 backalphakappa = ParameterOfBackGaussian(roughness);
        float alpha = backalphakappa.x;
        //magic number, to avoid 0 and large 1/kappa
        float kappa = max(1.0f,roughness*50.0f);
        
        //Projected normal
        float2 projNormal = ProjectNormal(backNormal);

        //Moment
        float3 moment = float3(1/kappa + projNormal.x*projNormal.x,
                               projNormal.x*projNormal.y,
                               1/kappa + projNormal.y*projNormal.y );
        //pack to [0,1]
        float2 lean1 = EncodeNormal2d(projNormal);
        float3 lean2 = EncodeNormal3d(float3(moment.x,moment.y,moment.z)/normalMax);

        output.Lean1 = float4(lean1,0,alpha);
        output.Lean2 = float4(lean2,0);

        return output;
    }

}


//--------------------------------------------------------------//
// Merge
//--------------------------------------------------------------//
struct Merge_Vertex_Shader_VS_INPUT 
{
    float4 Position : POSITION0;
};

struct Merge_Vertex_Shader_VS_OUTPUT 
{
    float4 Position : POSITION0;
    float2 texCoord : TEXCOORD0;
};

Merge_Vertex_Shader_VS_OUTPUT Merge_Vertex_Shader_vs_main( Merge_Vertex_Shader_VS_INPUT Input )
{
    Merge_Vertex_Shader_VS_OUTPUT Output;

    Output.Position = Input.Position;//mul( Input.Position, Merge_Vertex_Shader_matViewProjection );
    Output.texCoord = float2((Output.Position.x/Output.Position.w + 1)/2 + fInverseViewportWidth/2
        ,(-Output.Position.y/Output.Position.w + 1)/2 + fInverseViewportHeight/2);

    return( Output );

}

sampler2D texmerge = sampler_state
{   
    Texture = (back_MergeTex);
};


struct Merge_Pixel_Shader_PS_OUTPUT
{
    float4 color : COLOR0;
};
//TODO: clean all the names!
Merge_Pixel_Shader_PS_OUTPUT Merge_Pixel_Shader_ps_main(Merge_Vertex_Shader_VS_OUTPUT input)
{ 
    Merge_Pixel_Shader_PS_OUTPUT output;

    float2 f= float2(fInverseViewportWidth,fInverseViewportHeight);
    float4 merged = (tex2D( texmerge, input.texCoord + f*offsets[0] ));
    merged.xy = DecodeNormal2d(merged.xy);

    float4 mergedNormal = RestoreProjectedNormal(merged.xy);
    float powerAdjust = 0;
    if(merged.z > 0.000001f)
        ++powerAdjust;
    //Merge for every texture
    for( int i = 1; i < 4; i++ )
    {
        float4 newGaussian = (tex2D( texmerge, input.texCoord + f*offsets[i] ));
        newGaussian.xy = DecodeNormal2d(newGaussian.xy);
        //Mu1 and mu2
        float4 mu = mergedNormal;
        float4 newMu =  RestoreProjectedNormal( newGaussian.xy);
        float4 tempMu = 0;
        float sumAlpha = merged.z + newGaussian.z;
        if(newGaussian.z > 0.000001f)
            ++powerAdjust;
        //If one of the mu is INVALID
        if(mu.w < 0.0001f || newMu.w< 0.0001f)
        {
            tempMu = mu  + newMu ;//mu.w+newMu.w;
        }
        else 
        {
            if(sumAlpha < 0.00001f)
            {
                tempMu = (mu  + newMu )/2.0f;
            }
            else
            {
                tempMu =  merged.z/sumAlpha*mu + (1-merged.z/sumAlpha)*newMu;

            }
        }

        if(merged.w<0.00001f || newGaussian.w<0.00001f 
            || merged.z<0.00001f || newGaussian.z<0.00001f)
        {
            merged.w = max(merged.w,newGaussian.w);
        }
        else
        {
            //TODO: this merge formula might not be good!
            merged.w = sumAlpha/(merged.z/merged.w + newGaussian.z/newGaussian.w + merged.z*newGaussian.z*dot(mu-newMu,mu-newMu)/2.0f);
            //merged.w = merged.z/sumAlpha/merged.w + (1-merged.z/sumAlpha)/newGaussian.w;

            //merged.w = 2.0f*sumAlpha/(merged.z/merged.w + newGaussian.z/newGaussian.w);//(merged.w + newGaussian.w)/2.0f;//
        }
        merged.z = (merged.z + newGaussian.z);
        mergedNormal = tempMu; //notice this difference is [-1, 1]


    }
    //Notice that there are power adjust here
    //if(powerAdjust > 0.000001f )
    //    merged.z = merged.z/powerAdjust;
    merged.z = merged.z/4;
    merged.xy = EncodeNormal2d(ProjectNormal(mergedNormal.xyz));
    output.color = merged;
    return output;

}

//--------------------------------------------------------------//
// LeanDownsample
//--------------------------------------------------------------//
struct LeanDownsample_Vertex_Shader_VS_INPUT 
{
    float4 Position : POSITION0;
};

struct LeanDownsample_Vertex_Shader_VS_OUTPUT 
{
    float4 Position : POSITION0;
    float2 texCoord : TEXCOORD0;
};

LeanDownsample_Vertex_Shader_VS_OUTPUT LeanDownsample_Vertex_Shader_vs_main( LeanDownsample_Vertex_Shader_VS_INPUT Input )
{
    LeanDownsample_Vertex_Shader_VS_OUTPUT Output;

    Output.Position = Input.Position;//mul( Input.Position, LeanDownsample_Vertex_Shader_matViewProjection );
    Output.texCoord = float2((Output.Position.x/Output.Position.w + 1)/2 + fInverseViewportWidth/2
        ,(-Output.Position.y/Output.Position.w + 1)/2 + fInverseViewportHeight/2);

    return( Output );

}

struct LeanDownsample_Pixel_Shader_PS_OUTPUT
{
    float4 color : COLOR0;
};
//TODO: clean all the names!
LeanDownsample_Pixel_Shader_PS_OUTPUT LeanDownsample_Pixel_Shader_ps_main(LeanDownsample_Vertex_Shader_VS_OUTPUT input)
{ 
    LeanDownsample_Pixel_Shader_PS_OUTPUT output;

    //Simply downsample
    float2 f= float2(fInverseViewportWidth,fInverseViewportHeight);
    float4 merged = 0;

    //Linear interpolation
    for( int i = 0; i < 4; i++ )
    {
        merged += (tex2D( texmerge, input.texCoord + f*offsets[i] ));
    }

    //
    output.color = merged/4.0f;
    return output;

}
#endif